module exa ( 
	/* Inputs */
	start, reset, i_x, i_y, i_u, i_dx,
	/* Outputs */
	o_x, o_y, o_u, comp
	);
	
	input start;
	input reset;
	input [31:0] i_x;
	input [31:0] i_y;
	input [31:0] i_u;
	input [31:0] i_dx;
	output [31:0] o_x;
	output [31:0] o_y;
	output [31:0] o_u;
	output comp;

	/* Stage 0 */
	wire nreq0;
	wire [31:0] x_out, y_out, u_out, dx_out;
	stage0	stage0	(.reset(reset), .preq(start), .nreq(nreq0), .in1(i_x), .in2(i_y), .in3(i_u), .in4(i_dx),
							.out1(x_out), .out2(y_out), .out3(u_out), .out4(dx_out));

	/* Stage 1 */
	wire nreq1;
	wire [31:0] st1_y_out, st1_u_out, st1_dx_out;
	wire [31:0] add0_reg_out, mul0_reg_out, mul1_reg_out, mul2_reg_out;	
	stage1	stage1	(.reset(reset), .preq(nreq0), .nreq(nreq1), .in1(x_out), .in2(y_out), .in3(u_out), .in4(dx_out),
							.out1(st1_y_out), .out2(st1_u_out), .out3(st1_dx_out), .out4(add0_reg_out), .out5(mul0_reg_out), 
							.out6(mul1_reg_out), .out7(mul2_reg_out));
	
	/* Stage 2 */
	wire nreq2;
	wire [31:0] st2_u_out, st2_add0_reg_out;
	wire [31:0] add1_reg_out, mul3_reg_out, mul4_reg_out;		
	stage2	stage2	(.reset(reset), .preq(nreq1), .nreq(nreq2), .in1(st1_y_out), .in2(st1_u_out), .in3(st1_dx_out), 
							.in4(add0_reg_out), .in5(mul0_reg_out), .in6(mul1_reg_out), .in7(mul2_reg_out),
							.out1(st2_u_out), .out2(add1_reg_out), .out3(mul3_reg_out), .out4(mul4_reg_out), .out5(st2_add0_reg_out)); 
	
	/* Stage 3 */
	wire nreq3;
	wire [31:0] sub0_reg_out, st3_mul4_reg_out, st3_add0_reg_out, st3_add1_reg_out;	
	stage3	stage3	(.reset(reset), .preq(nreq2), .nreq(nreq3), .in1(st2_u_out), .in2(add1_reg_out), .in3(mul3_reg_out), .in4(mul4_reg_out), .in5(st2_add0_reg_out),
							.out1(sub0_reg_out), .out2(st3_mul4_reg_out), .out3(st3_add0_reg_out), .out4(st3_add1_reg_out));
								
	/* Stage 4 */
	stage4	stage4	(.reset(reset), .preq(nreq3), .nreq(comp), .in1(sub0_reg_out), .in2(st3_mul4_reg_out), .in3(st3_add0_reg_out), .in4(st3_add1_reg_out), 
							.out1(o_u), .out2(o_x), .out3(o_y));
	

endmodule
